<?php

/**
 * ajax action
 */
function ea_ajax(){
	if($_GET['action'] == 'ea_monthly_ajax') {
		$author_id = $_GET["author_id"];
		$year = $_GET["year"];

		$argsf = $_GET["args"];
		$args = str_replace('|', '&', $argsf);
		$args = str_replace('--', '=', $args);

		echo ea_get_archives($args, $year, $author_id, true);
		die();
	}
}
add_action('init', 'ea_ajax');

/**
 * print monthly archives and the filters
 * @param args		the arguements string
 * @param year		year filter
 * @param author	author ID filter
 */
function wp_easyarchives($args='', $year='', $author_id='') {
	echo '<div id="easy-archives" class="easy-archives">' . ea_get_archives($args, $year, $author_id) . '</div>';
}

/**
 * print monthly archives widget
 * @param args		the arguements string
 */
function wp_easyarchives_widget($args='') {
	$args = ea_parse_args($args);
	echo '<div class="easy-archives">';
	echo ea_get_monthly_archives('', '', $args['mode'], ($args['page']=='true'), $args['limit']);
	echo '</div>';
}

/**
 * format arguements
 * @param args		the arguements string
 * @return			formatting arguement array
 */
function ea_parse_args($args) {

	$defaults = array(
		'limit' => 0,
		'mode' => 'last',
		'page' => 'false',
	);

	$args = wp_parse_args($args, $defaults);

	if ($args['limit'] < 1) {
		$args['limit'] = 0;
	}
	if ($args['mode'] != 'last' && $args['mode'] != 'none') {
		$args['mode'] = '';
	}
	if ($args['page'] != 'true') {
		$args['page'] = 'false';
	}

	return $args;
}

/**
 * retrieve the monthly archives and the filters
 * @param args		the arguements string
 * @param year		year filter
 * @param author_id	author ID filter
 * @param ajax		AJAX action?
 * @return			monthly archives and filters
 */
function ea_get_archives($args, $year, $author_id, $ajax=false) {
	$argsf = str_replace('=', '--', $args);
	$argsf = str_replace('&', '|', $argsf);

	$args = ea_parse_args($args);

	$filter = '<div class="filter">';
	$filter .= ea_get_year_select($year, $argsf, ($args['page']=='true'));
	$filter .= ea_get_author_select($author_id, $argsf);
	$filter .= ea_get_expandall_button();
	$filter .= ea_get_collapseall_button();
	$filter .= '</div>';

	if ($ajax) {
		$limit = 0;
	} else {
		$limit = $args['limit'];
	}
	$archives = ea_get_monthly_archives($year, $author_id, $args['mode'], ($args['page']=='true'), $limit);

	return $filter . $archives;
}

/**
 * retrieve the monthly archives
 * @param year		year filter
 * @param author	author ID filter
 * @param mode		display mode
 * @param page		show page items?
 * @return			monthly archives
 */
function ea_get_monthly_archives($year='', $author_id='', $mode, $page, $limit) {
	global $month, $wpdb;

	// author id
	$sql_author = '';
	if ($author_id) {
		$sql_author = " AND post_author = " . $author_id;
	}

	// year
	$sql_year = '';
	if ($year) {
		$sql_year = " AND YEAR(post_date) = '" . $year . "'";
	}

	// pages
	$sql_page = " AND post_type = 'post'";
	if ($page) {
		$sql_page = " AND (post_type = 'post' OR post_type = 'page')";
	}

	// private posts
	global $user_ID;
	$sql_private = " AND (post_status != 'private' OR post_author = '" . $user_ID . "')";

	// where
	$where = " WHERE post_date < '" . current_time('mysql') . "' AND post_status != 'trash' AND post_status != 'draft' AND post_status != 'auto-draft'" . $sql_year . $sql_author . $sql_private . $sql_page;

	// orderby
	$orderby = " ORDER BY post_date DESC";

	// limit
	$sql_limit = "";
	if ($limit > 0) {
		$sql_limit = " LIMIT " . $limit;

		// get month sum
		$post_query = "SELECT COUNT(ID) FROM " . $wpdb->posts . $where . "GROUP BY DATE_FORMAT(post_date, '%Y%m')" . $orderby . $sql_limit;
		$month_counts = $wpdb->get_col($post_query);
		$month_sum = array_sum($month_counts);

		if ($month_sum > 0) {
			$sql_limit = " LIMIT " . $month_sum;
		}
	}

	// get posts
	$post_query = "SELECT *, YEAR(post_date) AS year, MONTH(post_date) AS month FROM " . $wpdb->posts . $where . $orderby . $sql_limit;
	$post_set = $wpdb->get_results($post_query);

	$monthly_archives = '<div class="monthly-archives">';
	if ($post_set) {
		// monthly item
		$monthly_item = '';
		// date of previous monthly item
		$pre_date = '';
		// the count of daily item
		$post_count = 0;
		$post_count_symbol = '%POSTCOUNT%';

		// loop through each month
		foreach($post_set as $post) {
			// get post date
			$date = get_the_time(__('F Y', 'wp-easyarchives'), $post);
			// if the date is diff, close previous monthly item
			if($date != $pre_date) {

				// when the first monthly item
				if($pre_date != '') {
					// set post count
					$monthly_item = str_replace($post_count_symbol, $post_count, $monthly_item);
					// ending of previous monthly item
					$monthly_item .= '</ul>';
					$monthly_item .= '</div>';
					// push previous monthly item to archives
					$monthly_archives .= $monthly_item;
				}

				// get the url of monthly archive
				$url = get_month_link($post->year , $post->month);
				// get status
				$display = 'open';
				$button = 'open-button';
				if ($mode == 'none' || ($pre_date != '' && $mode == 'last')) {
					$display = 'closed';
					$button = 'closed-button';
				}

				// begining of monthly item
				$monthly_item = '<div class="month">'; // new item
				$monthly_item .= '<div class="title">';
				$monthly_item .= '<a class="' . $button . '" href="javascript:void(0);" onclick="EAJS.toggle(this);"></a>';
				$monthly_item .= '<a class="detail" href="' . $url . '" title="' . sprintf(__('Show detailed results for %1$s', 'wp-easyarchives'), $date) . '">' . $date . '</a>';
				$monthly_item .= '<em>(' . sprintf(__('%1$s posts', 'wp-easyarchives'), $post_count_symbol) . ')</em>';
				$monthly_item .= '<div class="fixed"></div></div>';
				$monthly_item .= '<ul class="' . $display . '">';

				// current date becomes the previous date 
				$pre_date = $date;
				// reset post count
				$post_count = 0;
			}

			// get post title
			$post_title = $post->post_title;
			if ($post_title) {
				$title = wptexturize(strip_tags($post_title));
			} else {
				$title = $post->ID;
			}
			// get title tip
			$tip = sprintf(__('View this post, &quot;%1$s&quot;', 'wp-easyarchives'), wp_specialchars($title, 1));
			// get post url
			$url = get_permalink($post->ID);

			// begining of this daily item and post link
			$daily_item = '<li>' . mysql2date('d', $post->post_date) . ': ' . '<a href="' . $url .'" title="' . $tip . '">' . $title . '</a>';
			// show comment count
			$comment_count = $wpdb->get_var("SELECT COUNT(comment_id) FROM $wpdb->comments WHERE comment_post_ID=" . $post->ID . " AND comment_approved='1'");
			$daily_item .= '<em>(' . $comment_count . ')</em>';
			// ending of this daily item
			$daily_item .= '</li>';

			// push this daily item to its monthly item
			$monthly_item .= $daily_item;

			// increase the count of posts
			$post_count++;
		}

		// set post count
		$monthly_item = str_replace($post_count_symbol, $post_count, $monthly_item);
		// ending of last monthly item
		$monthly_item .= '</ul>';
		$monthly_item .= '</div>';
		// push last monthly item to archives
		$monthly_archives .= $monthly_item;
	}

	$monthly_archives .= '</div>';
	return $monthly_archives;
}

/**
 * retrieve list of years.
 * @param inclide_pages		include pages?
 * @return					years
 */
function ea_get_years($inclide_pages) {
	global $wpdb, $user_ID;

	// include pages
	$includes = " AND post_type='post'";
	if ($inclide_pages) {
		$includes = " AND (post_type = 'post' OR post_type = 'page')";
	}

	// private posts
	$sql_private = " AND (post_status != 'private' OR post_author = '" . $user_ID . "')";

	$years_query = "SELECT DISTINCT YEAR(post_date) AS year FROM $wpdb->posts WHERE post_date < '" . current_time('mysql') . "' " . $includes . $sql_private . " ORDER BY post_date DESC";
	$years = $wpdb->get_results($years_query);

	if (!$years || count($years) <= 1) {
		return false;
	}

	return $years;
}

/**
 * retrieve list of users who have writed a post
 * @return				author ids
 */
function ea_get_authors() {
	global $wpdb;

	$authors_query = "SELECT DISTINCT U.ID AS id, U.display_name AS name FROM $wpdb->users U, $wpdb->posts P WHERE U.ID = P.post_author ORDER BY id ASC";
	$authors = $wpdb->get_results($authors_query);

	if (!$authors || count($authors) <= 1) {
		return false;
	}

	return $authors;
}

/**
 * retrieve list of years.
 * @param selected_year		which year is selected?
 * @param args				javascript arguements string
 * @param inclide_pages		include pages?
 * @return					the years select element
 */
function ea_get_year_select($selected_year, $args, $inclide_pages) {
	$years = ea_get_years($inclide_pages);
	if (!$years) {
		return false;
	}

	$has_selected = false;
	foreach ($years as $year) {
		$year_options .= '<option value="' . $year->year . '" ' . ($year->year==$selected_year ? "selected=\"selected\"" : "") . '>' . $year->year . '</option>';
	}
	if ($has_selected) {
		$all_option .= '<option value="">All</option>';
	} else {
		$all_option .= '<option value="" selected="selected">All</option>';
	}

	$select = '<span class="ea_year">' . __('Year: ', 'wp-easyarchives') . '</span>';
	$select .= '<select class="ea_select" onchange="EAJS.changeYear(\'' . get_bloginfo('siteurl') . '\',\'' . $args . '\',this,\'' . __('loading', 'wp-easyarchives') . '\');">';
	$select .= $all_option . $year_options;
	$select .= '</select>';

	return $select;
}

/**
 * retrieve list of years.
 * @param selected_author	which author is selected?
 * @param args				javascript arguements string
 * @return					the authors select element
 */
function ea_get_author_select($selected_author, $args) {
	$authors = ea_get_authors();
	if (!$authors) {
		return false;
	}

	$has_selected = false;
	foreach ($authors as $author) {
		$author_options .= '<option value="' . $author->id . '" ' . ($author->id==$selected_author ? "selected=\"selected\"" : "") . '>' . $author->name . '</option>';
	}
	if ($has_selected) {
		$all_option .= '<option value="">All</option>';
	} else {
		$all_option .= '<option value="" selected="selected">All</option>';
	}

	$select = '<span class="ea_author">' . __('Author: ', 'wp-easyarchives') . '</span>';
	$select .= '<select class="ea_select" onchange="EAJS.changeAuthor(\'' . get_bloginfo('siteurl') . '\',\'' . $args . '\',this,\'' . __('loading', 'wp-easyarchives') . '\');">';
	$select .= $all_option . $author_options;
	$select .= '</select>';

	return $select;
}

/**
 * retrieve 'expand all' button.
 * @return					the 'expand all' button element
 */
function ea_get_expandall_button() {
	$button = '<input class="ea_button" type="button" value="' . __('Expand All', 'wp-easyarchives') . '" onclick="EAJS.expandAll();" />';
	return $button;
}

/**
 * retrieve 'collapse all' button.
 * @return					the 'collapse all' button element
 */
function ea_get_collapseall_button() {
	$button = '<input class="ea_button" type="button" value="' . __('Collapse All', 'wp-easyarchives') . '" onclick="EAJS.collapseAll();" />';
	return $button;
}

?>